
 
const nsIConsoleService = Components.interfaces.nsIConsoleService;
const nsIAutoCompleteSearch = Components.interfaces.nsIAutoCompleteSearch;
const nsIAutoCompleteResult = Components.interfaces.nsIAutoCompleteResult;
const nsISupports = Components.interfaces.nsISupports;

const ACSEARCHPROXY_CONTRACTID = '@mozilla.org/autocomplete/search;1?name=scProxy';
const ACSEARCHPROXY_CID = Components.ID('{64896643-3859-4BDA-997E-5718725CE2A2}');
const ACSEARCHPROXY_IID = Components.interfaces.scIAutoCompleteSearchProxy;



function scAutoCompleteSearchProxy() {
}


scAutoCompleteSearchProxy.prototype = {
	fProviders : {},
	fCounter : 1,
	fLastProviderCalled : null,
	
	/* */
    startSearch: function(pSearchString, pSearchParam, pPreviousResult, pListener) {
    	if(pSearchParam) {
    		var vIdx = pSearchParam.indexOf(":");
    		var vIdProvider = vIdx > 0 ? pSearchParam.substring(0, vIdx) : pSearchParam;
    		this.fLastProviderCalled = this.fProviders[vIdProvider];
    		if(this.fLastProviderCalled) {
    			this.fLastProviderCalled.startSearch(pSearchString, vIdx > 0 ? pSearchParam.substring(vIdx+1) : null, pPreviousResult, pListener);
    			return;
    		}
    	}

    	pListener.onSearchResult(this, {
            QueryInterface: function(pIID) {
              if (pIID.equals(nsIAutoCompleteResult) || pIID.equals(nsISupports))
                return this;
              throw Components.results.NS_NOINTERFACE;
            },
            results: [],
            defaultIndex: 0,
            errorDescription: null,
            matchCount: 0,
            searchResult: Components.interfaces.nsIAutoCompleteResult.RESULT_SUCCESS,
            searchString: pSearchString,
            getCommentAt: function(pIndex) {},
            getStyleAt: function(pIndex) {},
            getValueAt: function(pIndex) {},
            removeValueAt: function(pRowIndex, pRemoveFromDb) {}
         });
    	
    },
    
    /* */
    stopSearch: function() {
    	if(this.fLastProviderCalled) this.fLastProviderCalled.stopSearch();
    },
    
    registerProvider: function(pProvider) {
    	if(pProvider == null) return;
    	var vId = "a" + (this.fCounter++);
    	this.fProviders[vId] = pProvider;
    	return vId;
    },
    
    unregisterProvider: function(pIdProvider) {
    	if(pIdProvider == null) return;
    	delete this.fProviders[pIdProvider];
    },
    
    QueryInterface: function(pIID) {
            if (!pIID.equals(ACSEARCHPROXY_IID) &&
	            !pIID.equals(nsIAutoCompleteSearch) &&
                !pIID.equals(nsISupports))
                    throw Components.results.NS_ERROR_NO_INTERFACE;
            return this;
    }
}



function debug(pText){
	var vConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(nsIConsoleService);
	vConsoleService.logStringMessage(pText);
}



var scModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(ACSEARCHPROXY_CID, 
                                        "scAutoCompleteSearchProxySvc JS component", 
                                        ACSEARCHPROXY_CONTRACTID, 
                                        fileSpec, 
                                        location,
                                        type);
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(ACSEARCHPROXY_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return scAutoCompleteSearchProxySvcFactory;
    },

    canUnload: function(compMgr) { return true; }
};


var scAutoCompleteSearchProxySvcFactory = {
    createInstance: function(outer, iid) {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
    
        if (!iid.equals(ACSEARCHPROXY_IID) &&
            !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;

        return new scAutoCompleteSearchProxy();
    }
}


function NSGetModule(comMgr, fileSpec) { return scModule; }

